<?php
namespace Models;

// Модель для работы с Фискальным регистратором
class Fr extends \DB\SQL\Mapper{
	public $db, $f3, $printer, $sets, $fr, $ticket_type=array('1'=>0, '2'=>1, '128'=>128, '130'=>130), $api_data;

	public function __construct($db, $f3, $printer='kkm', $open=true) {
		parent::__construct($db, 'fr_settings'); // подключаемся к таблице с настройками
		$this->f3=$f3;
		$this->db=$db;
		$this->receipt=array();
		$this->discount_sum=0;
		$this->printer=$printer;
		$this->api_data=$this->f3->get('api_data');
		$this->sets=$this->get_fr();
		if($this->sets['active']==1 && $this->sets['driver']!=''  && $open===true) {
			if(file_exists($this->sets['driver'].'/server.py')) {
				$this->fr=new Fr_python($f3, $this->sets);
			}
			elseif(file_exists($this->sets['driver'].'/server.php')) {
				$class=explode('/', $this->sets['driver']);
				$class=array_reverse($class);
				$model='\Models\\'.$class[0].'_Fr_php';
				if(!class_exists($model, false)) {
					require_once($this->sets['driver'].'/server.php');
				}
				$file='/server.config';
				if($printer=='pre' || $printer=='kitchen' || $printer=='bar') {
					if(file_exists($this->sets['driver'].'/'.$printer.'_server.config')) {
						$file='/'.$printer.'_server.config';
					}
				}
				$config=parse_ini_file($this->sets['driver'].$file);
				$this->fr=new $model($f3, $this->sets, $config);
			}
			else {
				echo 'Ошибка';
			}
		}
	}

	// Список драйверов
	function get_drivers() {
		$drivers=array();
		$printers_dir=$this->f3->get('printers_dir');
		$dirs=scandir($printers_dir); // Ищем все драйвера
		if($dirs) {
			foreach ($dirs as $d) {
				$folder=$printers_dir.$d;
				$file=$folder.'/server.config';
				if(is_dir($folder) && file_exists($file)) {
					$config=parse_ini_file($file);
					if($config) {
						$drivers[$folder]=array(
							'title'=>$config['ServerName'],
							'path'=>$folder
						);
					}
				}
			}
		}

		return $drivers;
	}

	// Настройки ФР
	function get_fr() {
		$this->reset();
		$this->load(array("printer=? AND rm=?", array($this->printer, $this->api_data['rm_id'])), array('limit'=>1));
		$sets=$this->cast();
		$sets['pays_type']=json_decode($sets['pays_type'], true);
		if(!$sets['vat_type']) {
			$sets['vat_type']='["4","1","5","2","6","3"]';
		}
		$sets['vat_type']=json_decode($sets['vat_type'], true);
		return $sets;
	}

	// Сохраняем настройки ФР
	function save_fr($post) {
		if($this->printer=='kkm' && in_array(0, $post['vat']) && $post['active']==1) {
			return array(
				'success'=>false,
				'txt'=>'vat_not_null'
			);
		}
		if($this->printer=='kkm') {
			$post['pays_type']=json_encode($post['pays']);
			$post['vat_type']=json_encode($post['vat']);
			unset($post['pays'], $post['vat']);
		}
		else {
			$post['pays_type']='';
			$post['vat_type']='';
		}
		if($post['fix_user']) {
			$post['fix_user']=trim($post['fix_user']);
		}
		if($post['fix_inn']) {
			$post['fix_inn']=str_replace(' ', '', $post['fix_inn']);
		}
		if($post['free_title']) {
			$post['free_title']=trim($post['free_title']);
		}
		if(!$post['rm']) {
			$post['rm']=$this->api_data['rm_id'];
		}
		$this->reset();
		$this->get_fr();
		$post['printer']=$this->printer;
		$this->copyFrom($post);
		if($this->save()===false) {
			return array(
				'success'=>false,
				'txt'=>'db_error'
			);
		}
		return array(
			'success'=>true,
			'txt'=>'save_successful'
		);
	}

	function ascii_str($str) {
		$alphabet=array(
			0=>array(" #### ","##  ##","##  ##","##  ##"," #### ", "      "),
			1=>array("   ##", " ####", "## ##", "   ##", "   ##", "     "),
			2=>array(" #### ","##  ##","   ## ","  ##  ","######", "      "),
			3=>array(" #### ","##  ##","   ## ","##  ##"," #### ", "      "),
			4=>array("##  ##","##  ##","######","    ##","    ##", "      "),
			5=>array("######","##    ","##### ","    ##","##### ", "      "),
			6=>array(" #####","##    ","##### ","##  ##"," #### ", "      "),
			7=>array("######","   ## ","  ##  "," ##   ","##    ", "      "),
			8=>array(" #### ","##  ##"," #### ","##  ##"," #### ", "      "),
			9=>array(" #####","##  ##"," #####","    ##"," #### ", "      "),
			'А'=>array("  ##  ", " #### ", "##  ##", "######", "##  ##", "      "),
			'Б'=>array("######", "##    ", "######", "##  ##", "######", "      "),
			'В'=>array("##### ", "##  ##", "##### ", "##  ##", "##### ", "      "),
			'Г'=>array("######", "##    ", "##    ", "##    ", "##    ", "      "),
			'Д'=>array("  #####", " ##  ##", " ##  ##", "########", "##    ##", "        "),
			'Е'=>array("######", "##    ", "##### ", "##    ", "######", "      "),
			'Ё'=>array(" #  # ", "######", "##    ", "##### ", "##    ", "######"),
			'Ж'=>array("## ## ##", "## ## ##", " ###### ", "## ## ##", "## ## ##", "        "),
			'З'=>array(" #### ", "##  ##", "   ## ", "##  ##", " #### ", "      "),
			'И'=>array("##  ##", "## ###", "######", "### ##", "##  ##", "      "),
			'К'=>array("##  ##", "## ## ", "####  ", "## ## ", "##  ##", "      "),
			'Л'=>array("  ##  ", " #### ", "##  ##", "##  ##", "##  ##", "      "),
			'М'=>array("##     ##", "###   ###", "## ### ##", "##  #  ##", "##     ##", "         "),
			'Н'=>array("##  ##", "##  ##", "######", "##  ##", "##  ##", "      "),
			'О'=>array(" #### ", "##  ##", "##  ##", "##  ##", " #### ", "      "),
			'П'=>array("######", "##  ##", "##  ##", "##  ##", "##  ##", "      "),
			'Р'=>array("##### ", "##  ##", "##### ", "##    ", "##    ", "      "),
			'С'=>array(" #### ", "##  ##", "##    ", "##  ##", " #### ", "      "),
			'Т'=>array("######", "  ##  ", "  ##  ", "  ##  ", "  ##  ", "      "),
			'У'=>array("##   ##", " ## ## ", "  ###  ", " ##    ", "##     ", "       "),
			'Ф'=>array(" ###### ", "## ## ##", " ###### ", "   ##   ", "   ##   ", "        "),
			'Х'=>array("##   ##", " ## ## ", "  ###  ", " ## ## ", "##   ##", "       "),
			'Ц'=>array("##  ##  ", "##  ##  ", "##  ##  ", "##  ##  ", "########", "      ##"),
			'Ч'=>array("##  ##", "##  ##", " #####", "    ##", "    ##", "      "),
			'Ш'=>array("## ## ##", "## ## ##", "## ## ##", "## ## ##", "########", "        "),
			'Щ'=>array("## ## ##  ", "## ## ##  ", "## ## ##  ", "## ## ##  ", "##########", "        ##"),
			'Ъ'=>array("####    ", "  ##    ", "  ##### ", "  ##  ##", "  ##### ", "        "),
			'Ы'=>array("##     ##", "##     ##", "#####  ##", "##  ## ##", "#####  ##", "         "),
			'Ь'=>array("##    ", "##    ", "##### ", "##  ##", "##### ", "      "),
			'Э'=>array(" #### ", "##  ##", "   ###", "##  ##", " #### ", "      "),
			'Ю'=>array("##  #### ", "## ##  ##", "#####  ##", "## ##  ##", "##  #### ", "        "),
			'Я'=>array("  #####", " ##  ##", "  #####", " ##  ##", "##   ##", "       "),
			'A'=>array("  ##  ", " #### ", "##  ##", "######", "##  ##", "      "),
			'B'=>array("##### ", "##  ##", "##### ", "##  ##", "##### ", "      "),
			'C'=>array(" #### ", "##  ##", "##    ", "##  ##", " #### ", "      "),
			'D'=>array("##### ", "##  ##", "##    ", "##  ##", "##### ", "      "),
			'E'=>array("######", "##    ", "##### ", "##    ", "######", "      "),
			'F'=>array("######", "##    ", "##### ", "##    ", "##    ", "      "),
			'G'=>array(" #####", "##    ", "##  ##", "##   #", " #### ", "      "),
			'H'=>array("##  ##", "##  ##", "######", "##  ##", "##  ##", "      "),
			'I'=>array("####", " ## ", " ## ", " ## ", "####", "    "),
			'J'=>array("    ##", "    ##", "    ##", "##  ##", " #### ", "      "),
			'K'=>array("##  ##", "## ## ", "####  ", "## ## ", "##  ##", "      "),
			'L'=>array("##    ", "##    ", "##    ", "##    ", "######", "      "),
			'M'=>array("##     ##", "###   ###", "## ### ##", "##  #  ##", "##     ##", "         "),
			'N'=>array("##  ##", "### ##", "######", "## ###", "##  ##", "      "),
			'O'=>array(" #### ", "##  ##", "##  ##", "##  ##", " #### ", "      "),
			'P'=>array("##### ", "##  ##", "##### ", "##    ", "##    ", "      "),
			'R'=>array("##### ", "##  ##", "##### ", "## ## ", "##  ##", "      "),
			'S'=>array(" #####", "##    ", " #### ", "    ##", "##### ", "      "),
			'T'=>array("######", "  ##  ", "  ##  ", "  ##  ", "  ##  ", "      "),
			'U'=>array("##  ##", "##  ##", "##  ##", "##  ##", " #### ", "      "),
			'V'=>array("##    ##", "##    ##", " ##  ## ", "  ####  ", "   ##   ", "        "),
			'W'=>array("##     ##", "##  #  ##", "## ### ##", "###   ###", "##     ##", "         "),
			'X'=>array("##   ##", " ## ## ", "  ###  ", " ## ## ", "##   ##", "       "),
			'Y'=>array("##    ##", " ##  ## ", "   ##   ", "   ##   ", "   ##   ", "        "),
			'Z'=>array("######", "   ## ", "  ##  ", " ##   ", "######", "      ")
		);

		$p='';
		$symbols=array();
		$strlen=mb_strlen($str);
		for($sympos=0; $sympos<$strlen; $sympos++) {
			$symbols[]=$alphabet[mb_strtoupper(mb_substr($str, $sympos, 1))];
		}
		for($i=0; $i<6; $i++) {
			foreach($symbols as $n=>$s) {
				$p.=$s[$i];
				if($n<$strlen-1) {
					$p.=' ';
				}
			}
			$p.='#kkm_br#';
		}
		return $p;
	}

	// Открытие смены
	function open_session($print=1) {
		$user=$this->f3->get('authorization_user.short_name');
		$user_inn=$this->f3->get('authorization_user.inn');
		if($this->sets['fix_user']) {
			$user=$this->sets['fix_user'];
			$user_inn=$this->sets['fix_inn'];
		}
		if($user_inn) {
			$user.='{{user_inn}}'.$user_inn;
		}
		$strs="preitem;\n";
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		$strs.="preitem_end;\n";
		$strs.="open_session;".$print.";".$user.";\n";
		if(!$this->sets['type'] && $print==1) {
			$strs.="p;ОТКРЫТИЕ СМЕНЫ#kkm_br#".date("d-m-Y H:i").";\n";
		}
		//$strs.="p;#kkm_br#Касса: ".$this->api_data['rm_id']."#kkm_br#";
		//$strs.="Кассир ".$this->f3->get('authorization_user.short_name').";\n";
		$strs.="postitem;\n";
		if($this->sets['footer'] && $print==1) {

			$strs.=$this->sets['footer']."\n";
		}
		$strs.="postitem_end;\n";
		
		$this->send_ajax($strs);
	}

	// Закрытие смены
	function z($change, $print=1) {
		$user=$this->f3->get('authorization_user.short_name');
		$user_inn=$this->f3->get('authorization_user.inn');
		if($this->sets['fix_user']) {
			$user=$this->sets['fix_user'];
			$user_inn=$this->sets['fix_inn'];
		}
		if($user_inn) {
			$user.='{{user_inn}}'.$user_inn;
		}
		$strs="preitem;\n";
		if($print==1) {
			if($this->sets['header']) {
				$strs.=$this->sets['header']."\n";
			}
		}
		$strs.="preitem_end;\n";
		$strs.="z;".$print.";".$user.";\n";
		if(!$this->sets['type'] && $print==1) {
			$strs.=$this->report($change, 'z');
		}
		else {
			//$strs.="p;Касса: ".$this->api_data['rm_id']."#kkm_br#";
			//$strs.="Кассир ".$this->f3->get('authorization_user.short_name').";\n";
			$strs.="postitem;\n";
			if($this->sets['footer'] && $print==1) {
				$strs.=$this->sets['footer']."\n";
			}
			$strs.="postitem_end;\n";
		}
		$bt=new \Models\BankTerminal($this->db, $this->f3);
		$res=$bt->z();
		if(!$res['success']) {
			echo $res['txt'];
			exit();
		}
		elseif($res['txt']) {
			$slip=str_replace("\n", '', $res['txt']);
			$slip=str_replace("\r", '', $slip);
			$slip='p;'.preg_replace('/\<br(\s*)?\/?\>/i', "#kkm_br#", $slip).";\n";
			$this->send_ajax($slip);
		}
		$this->send_ajax($strs);
	}

	// X-отчет смены
	function x($change) {
		$strs="x;\n";
		if(!$this->sets['type']) {
			$strs.=$this->report($change, 'x');
		}
		else {
			//$strs.="p;Касса: ".$this->api_data['rm_id']."#kkm_br#";
			//$strs.="Кассир ".$this->f3->get('authorization_user.short_name').";\n";
			if($this->sets['footer']) {
				$strs.=$this->sets['footer']."\n";
			}
		}
		$bt=new \Models\BankTerminal($this->db, $this->f3);
		$res=$bt->x();
		if(!$res['success']) {
			echo $res['txt'];
			exit();
		}
		elseif($res['txt']) {
			$slip=str_replace("\n", '', $res['txt']);
			$slip=str_replace("\r", '', $slip);
			$slip='p;'.preg_replace('/\<br(\s*)?\/?\>/i', "#kkm_br#", $slip).";\n";
			$this->send_ajax($slip);
		}
		$this->send_ajax($strs);
	}

	function report($change, $type) {
		$types=array('x'=>'НУЛЕВОЙ ОТЧЕТ', 'z'=>'ОТЧЕТ С ГАШЕНИЕМ');
		$strs='';
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		$strs.="p;".$types[$type]."#kkm_br#";

		if($change) {
			$receipts=new \Models\Receipts($this->db, $this->f3);
			$report=$receipts->change_statistic($change['id']);
			$p=new \Models\Paymentmethods($this->db, $this->f3);
			$paymentmethods=$p->select_method();
			foreach ($paymentmethods as $p) {
				$pays[$p['id']]=array('title'=>$p['title'], 'cash'=>$p['cash']);
			}
			$strs.=date("d/m/Y H:i")."#kkm_br#";
			$strs.="Смена ".$change['id']."#kkm_br#";
			$strs.="Начало смены ".date("d/m/Y H:i", $change['time_start'])."#kkm_br#";
		}

		if($type=='z') {
			$strs.="Конец смены ".date("d/m/Y H:i");
		}
		$strs.="#kkm_br#";

		if($change) {
			$types=array(
				$this->f3->get('receipt_sales')=>'Продажи',
				$this->f3->get('receipt_return')=>'Возвраты',
				$this->f3->get('receipt_cashin')=>'Внесение',
				$this->f3->get('receipt_cashout')=>'Изъятие'
			);
			foreach ($report as $type => $r) {
				if(!$types[$type]) continue;
				$strs.="#kkm_br#".$types[$type]." = ".$r['count']."#kkm_br#";
				foreach ($pays as $i => $p) {
					$strs.="  ".$p['title']." = ".number_format(abs($r['pay'][$i]), 2, '.', '')."#kkm_br#";
					if($p['cash']==1) {
						if($type==$this->f3->get('receipt_return'))$total-=$r['pay'][$i];
						else $total+=$r['pay'][$i];
					}
				}
				$strs.="  Сумма = ".number_format(abs($r['sum']), 2, '.', '')."#kkm_br##kkm_br#";
			}
			$strs.="#kkm_br#Наличность в кассе = ".number_format($total, 2, '.', '');
			$strs.=";\n";
		}
		//$strs.="p;Касса: ".$this->api_data['rm_id']."#kkm_br#";
		//$strs.="Кассир ".$this->f3->get('authorization_user.short_name').";\n";
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		return $strs;
	}

	// Добавляем чек
	function ticket($t, $lines, $discount=0, $sales_receipts=0, $print=1, $egais=null, $partner_bonus=0, $partner_bonus_sum=0, $bonus_pay=0) {
		$pays=json_decode($t['payment'], true);
		if($this->f3->get('GET.continue_print')==1) {
			$this->f3->clear('GET.continue_print');
			$this->continue_print();
		}
		else {
			$bt=new \Models\BankTerminal($this->db, $this->f3);
			$e_paymethod=(int)$this->f3->get('e_paymethod');;
			foreach($pays as $pay) {
				if((int)$pay['method']==$e_paymethod && $pay['sum']>0) {
					$res=$bt->ticket($t['type'], $pay['sum']);
					if(!$res['success']) {
						echo $res['txt'];
						exit();
					}
					elseif($res['txt']) {
						$slip=str_replace("\n", '', $res['txt']);
						$slip=str_replace("\r", '', $slip);
						$slip='p;'.preg_replace('/\<br(\s*)?\/?\>/i', "#kkm_br#", $slip).";\n";
						$this->send_ajax($slip);
					}
				}
			}
		}
		if($this->f3->get('hold')) return true;
		$this->receipt=$t['receipt'];
		$user=$this->f3->get('authorization_user.short_name');
		$user_inn=$this->f3->get('authorization_user.inn');
		if($this->sets['fix_user']) {
			$user=$this->sets['fix_user'];
			$user_inn=$this->sets['fix_inn'];
		}
		if($user_inn) {
			$user.='{{user_inn}}'.$user_inn;
		}
		$vat_type=$this->f3->get('vat_type');
		$psr=1; // Признак способа расчета по умолчанию
		$ppr=1; // Признак предмета расчета по умолчанию
		$section=1; // Секция

		// Заголовок
		$strs="preitem;\n";
		if($this->sets['header'] && $print==1) {
			$strs.=$this->sets['header']."\n";
		}

		if($sales_receipts && $print==1) {
			$strs.="print_font;2;Товарный чек;\n";
		}
		$strs.="preitem_end;\n";
		// Товары
		$lines_str='';
		$discount_sum=0;
		$ticket_sum=0;
		$round_kop=0;
		foreach ($lines as $l) {
			if($l['price']==0 && $l['product_id']==0) {
				continue;
			}
			if($l['price']<0)  {
				$round_kop=1;
				continue;
			}
			if($l['loyalty']==0 && !$l['discount']) {
				$l['discount']=$discount;
			}
			if($l['discount']>$l['max_discount']) {
				$l['discount']=$l['max_discount'];
			}
			$title=mb_substr(str_replace(';', ',', $l['product_title']), 0, 100);
			$price=$l['price']+($l['price']*($l['vat']/100))*max(0, $l['vat_type']);
			$d_sum=(float) $price*((float) $l['discount']/100);
			$price-=$d_sum;
			if($l['line_discount']>0) {
				$price0=$l['price0']+($l['price0']*($l['vat']/100))*max(0, $l['vat_type']);
				$d_sum=(float) $price0*((float) $l['line_discount']/100);
			}
			$discount_sum+=$d_sum*$l['count'];
			if($bonus_pay>0) {
				$b=min($bonus_pay, $price);
				$price-=$b;
				$bonus_pay-=$b;
			}
			$price*=100;
			$price=round($price);
			$vt=$vat_type;
			if(!isset($l['ppr'])) $l['ppr']=$ppr;
			if(isset($l['other_info'])) {
				$other_info=json_decode($l['other_info'], true);
				if(isset($other_info['ppr'])) $l['ppr']=$other_info['ppr'];
			}
			if($l['vat_type']==0) $vt=$this->sets['vat_type'][0];
			elseif($l['vat_type']!=0 && $l['vat']==0) $vt=$this->sets['vat_type'][1];
			elseif($l['vat_type']==1 && $l['vat']>0 && $l['vat']<=10) $vt=$this->sets['vat_type'][2];
			elseif($l['vat_type']==-1 && $l['vat']>0 && $l['vat']<=10) $vt=$this->sets['vat_type'][3];
			elseif($l['vat_type']==1 && $l['vat']>10) $vt=$this->sets['vat_type'][4];
			elseif($l['vat_type']==-1 && $l['vat']>10) $vt=$this->sets['vat_type'][5];
			if($l['mark']!='') {
				$mark=json_decode($l['mark'], true);
				$i=1;
				if(!$l['pg'])$l['pg']='other';
				foreach ($mark as $m) {
					$count=1;
					if($i==count($mark))$count=$l['count']-$i+1;
					$i++;
					$lines_str.='smde;'.$title.'{{marking}}'.str_replace(';', '{{U+003B}}', ($m)).'{{marking}}'.$l['pg'].';'.$price.';'.round($count*1000).';'.$vt.';'.$section.';';
					if($this->sets['version']=="1.1" || $this->sets['version']=="1.2") {
						$lines_str.=$psr.';'.$l['ppr'].';';
					}
					$lines_str.="\n";
					//$strs.="set_tlv;1162;5;".$m.";\n";
				}
			}
			else {
				$lines_str.='smde;'.$title.';'.$price.';'.round($l['count']*1000).';'.$vt.';'.$section.';';
				if($this->sets['version']=="1.1" || $this->sets['version']=="1.2") {
					$lines_str.=$psr.';'.$l['ppr'].';';
				}
				$lines_str.="\n";
			}
			$ticket_sum+=$price*$l['count'];
		}
		//
		if($round_kop==1) {
			$total_sum=floor($ticket_sum/100)*100;
		}
		else {
			$total_sum=$ticket_sum;
		}
		$strs.='b;'.($this->ticket_type[$t['type']]).';'.$user.";".$print.";".$total_sum.';'.$ticket_sum.";\n"; // Открываем чек

		if($t['client']) {
			$strs.="set_tlv;1008;5;".$t['client'].";\n";
		}

		if($t['correct_type']) {
			$strs.="set_tlv;1173;1;".$t['correct_type'].";\n";
		}
		if($t['osn_date']) {
			$dates=explode('.', $t['osn_date']);
			$strs.="set_tlv;1178;3;".(gmmktime(0, 0, 0, $dates[1], $dates[0], $dates[2])).";\n";
		}
		if($t['osn_number']) {
			$strs.="set_tlv;1179;5;".$t['osn_number'].";\n";
		}

		$strs.=$lines_str;

		// оплата
		$strs.='tmde;';

		foreach ($pays as $p) {
			$mpay[$this->sets['pays_type'][$p['method']]]=$p['sum'];
		}

		$pay_count=5;

		for($i=0; $i<$pay_count; $i++) {
			if(!$mpay[$i])$mpay[$i]=0;
			$strs.=max(round($mpay[$i]*100), 0).';';
		}

		$strs.=$this->sets['sno'].';';
		$strs.="\n";
		$strs.="postitem;\n";
		if($egais && isset($egais['url']) && isset($egais['sign'])) {
			$strs.="p;".$egais['sign'].";\n";
			$strs.="qr;".$egais['url'].";\n";
			$strs.="p;".$egais['url'].";\n";
		}

		if($print==1) {
			if($this->sets['type']==0) {
				$strs.="p;Позиций#kkm_right#".count($lines);
				if($discount_sum>0) {
					$strs.="#kkm_br#Скидка#kkm_right#".number_format($discount_sum, 2, '.', '');
				}
				$strs.="#kkm_br#";
				$strs.="Касса: ".$this->api_data['rm_id'];
				$strs.="#kkm_br#".date("d.m.Y H:i")." N ".$t['id']."#kkm_br#";
				$strs.="Кассир#kkm_br#".$user.";\n";
			}
			if($partner_bonus!=0 || $partner_bonus_sum!=0) {
				$strs.="p;Начисленно баллов: ".$partner_bonus."#kkm_br#Всего баллов: ".$partner_bonus_sum.";\n";
			}
			if($this->sets['footer']) {
				$strs.=$this->sets['footer']."\n";
			}
		}
		$strs.="postitem_end;\n";
		$this->discount_sum=$discount_sum;
		return $this->send_ajax($strs);
	}

	// Внесение в кассу
	function cashin($change, $sum, $print_type=null) {
		if($this->f3->get('GET.continue_print')==1) {
			$this->f3->clear('GET.continue_print');
			$this->continue_print();
		}
		if($this->f3->get('hold')) return true;
		$user=$this->f3->get('authorization_user.short_name');
		$user_inn=$this->f3->get('authorization_user.inn');
		if($this->sets['fix_user']) {
			$user=$this->sets['fix_user'];
			$user_inn=$this->sets['fix_inn'];
		}
		if($user_inn) {
			$user.='{{user_inn}}'.$user_inn;
		}
		$strs="preitem;\n";
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		$strs.="preitem_end;\n";
		if($print_type!==0) {
			$strs.="imde;".($sum*100).";".$user.";\n";
		}
		if(!$this->sets['type'] || $print_type===0) { 
			// Заголовок
			if($sum>=0) {
				$strs.="p;ВНЕСЕНИЕ В КАССУ";
			}
			else {
				$strs.="p;ИЗЪЯТИЕ ИЗ КАССЫ";
			}
			$strs.="#kkm_br#";
			$strs.="Сумма = ".number_format(abs($sum), 2, '.', '')."#kkm_br#";
			//$strs.="Касса: ".$this->api_data['rm_id']."#kkm_br#";
			//$strs.="Кассир ".$this->f3->get('authorization_user.short_name')."#kkm_br#";
			$strs.="Смена ".$change.";\n";
			//$strs.=date("d-m-Y H:i").";\n";
		}
		/*else {
			$strs.="p;Касса: ".$this->api_data['rm_id']."#kkm_br#";
			$strs.="Кассир ".$this->f3->get('authorization_user.short_name').";\n";
		}*/
		// подвал
		$strs.="postitem;\n";
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		$strs.="postitem_end;\n";
		
		$this->send_ajax($strs);
	}

	// Обнуление не пробитого чека
	function reset_ticket() {
		$strs="g;";
		$this->send_ajax($strs);
		return array(
			'success'=>true,
			'txt'=>'ticket_reset'
		);
	}

	// Запрос полного статуса ККМ
	function status_kkm() {
		$strs="d;";
		$this->fr->send_ajax($strs, null, true);
	}

	// Продолжить печать, после аварийной остановки
	function continue_print() {
		$strs="continue_print;";
		$this->send_ajax($strs);
		return array(
			'success'=>true,
			'txt'=>'print_continue'
		);
	}

	// Отчет по секциям
	function rep_sections() {
		$strs="rep_sections;";
		$this->send_ajax($strs);
	}

	// Печать копии последнего чека
	function c() {
		$strs='';
		// Заголовок
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		$strs.="c;\n";
		// Подвал
		if($this->sets['footer']) {
			$strs.=$this->sets['footer']."\n";
		}
		//$strs.="p;Касса: ".$this->api_data['rm_id']."#kkm_br#";
		//$strs.="Кассир ".$this->f3->get('authorization_user.short_name').";\n";
		
		$this->send_ajax($strs);
	}

	// Печать пречека или товарного чека
	function print($receipt) {
		$this->receipt=$receipt;
		$strs='';
		// Заголовок
		$strs="preitem;\n";
		if($this->sets['header']) {
			$strs.=$this->sets['header']."\n";
		}
		$strs.="preitem_end;\n";

		if($this->printer!='kitchen' && $this->printer!='bar') {
			$strs.='p;';
			// Товарный чек или пречек
			if($receipt['time']>0) {
				$strs.='#kkm_center#ТОВАРНЫЙ ЧЕК#kkm_br#';
			}
			else {
				$strs.='#kkm_center#ПРЕДВАРИТЕЛЬНЫЙ ЧЕК#kkm_br#';
			}
			//$strs.=";\n";
		}
		else {
			$strs.='print_font;2;';
		}

		$sum=0;
		$discount_sum=0;
		$turn=0;
		// Позиции в чеке
		foreach ($receipt['receiptlines'] as $l) {
			// Цена
			if($l['loyalty']==0 && !$l['discount']) {
				$l['discount']=$receipt['discount'];
			}
			if($l['discount']>$l['max_discount']) {
				$l['discount']=$l['max_discount'];
			}
			$price=$l['price']+($l['price']*($l['vat']/100))*max(0, $l['vat_type']);
			$price=round($price, 2);

			$d_sum=$price*($l['discount']/100);
			$price-=$d_sum;
			if($l['line_discount']>0) {
				$price0=$l['price0']+($l['price0']*($l['vat']/100))*max(0, $l['vat_type']);
				$d_sum=(float) $price0*((float) $l['line_discount']/100);
			}
			$discount_sum+=$d_sum*$l['count'];
			$sum+=$price*$l['count']; // Сумма чека
			$l['product_title']=str_replace(';', ',', $l['product_title']);
			if($l['parent_id']>0) {
				$l['product_title']=' -'.$l['product_title'];
			}
			if($this->printer=='kitchen' || $this->printer=='bar') {
				if($l['turn']!=$turn) {
					$strs.='#kkm_br#--- Очередь '.$l['turn'].' ---#kkm_br#';
				}
				$turn=$l['turn'];
				$strs.='#kkm_br#';
				
				$strs.=$l['product_title'].'#kkm_br##kkm_right#X'.number_format($l['count'], 3, '.', '');
			}
			else {
				if((int)$this->sets['one_line']==1) {
					$strs.='#kkm_br#';
				}
				else {
					$strs.='p;';
				}
				$strs.=$l['product_title'].'#kkm_br##kkm_right#'.number_format($l['count'], 3, '.', '').'X'.number_format($price, 2, '.', '').'='.number_format($price*$l['count'], 2, '.', '');
			}

			// НДС товара
			if($l['vat_type']!=0) {
				$vp=(($l['price']/(100+$l['vat']*abs(min(0, $l['vat_type']))))*$l['vat'])*$l['count'];
				$vp=round($vp, 2);
				$vat[$l['vat']]+=$vp; // Сумма НДС по чеку
				if($this->printer!='kitchen' && $this->printer!='bar') {
					$strs.='#kkm_br#НДС#kkm_right#'.$l['vat'].'%='.number_format($vp, 2, '.', '').'#kkm_br#';
				}
			}
			$strs.='#kkm_br#';
			
		}

		if($this->printer!='kitchen' && $this->printer!='bar') {
			if((int)$this->sets['one_line']!=1) {
				$strs.="p;";
			}
			$strs.='#kkm_bold#ИТОГ#kkm_right#='.number_format($sum, 2, '.', '').'#kkm_br#';
		}

		// Если чек пробит
		if($this->printer!='kitchen' && $this->printer!='bar' && $receipt['time']>0 && $receipt['pays']) {
			$cash=0;
			// Оплата
			$strs.='ОПЛАТА#kkm_br#';
			foreach ($receipt['pays'] as $p) {
				$cash+=$p['sum'];
				$strs.=$p['title'].'#kkm_right#='.number_format(round($p['sum'], 2), 2, '.', '').'#kkm_br#';
			}
			// Сдача
			$strs.='Сдача#kkm_right#='.number_format($cash-$sum, 2, '.', '').'#kkm_br#';
			$sno=array('01'=>'ОСН','02'=>'УСН','04'=>'УСН','08'=>'ЕНВД','10'=>'ЕСХН ','20'=>'Патент');
			// СНО
			$strs.='СНО#kkm_right#'.$sno[$this->sets['sno']].'#kkm_br#';
			// НДС по чеку
			if($vat) {
				$strs.='НДС#kkm_br#';
				foreach ($vat as $p=>$v) {
					$strs.='#kkm_right#'.number_format($p, 2, '.', '').'% = '.number_format($v, 2, '.', '').'#kkm_br#';
				}
			}
			// Кассир и смена
			$strs.='Кассир#kkm_br#';
			$user=$receipt['user']['short_name'];
			if($this->sets['fix_user']) {
				$user=$this->sets['fix_user'];
			}
			$strs.=$user.'#kkm_br#';
			$strs.='Смена '.$receipt['changes'].'#kkm_br#';
		}

		if($this->printer!='kitchen' && $this->printer!='bar') {
			$strs.="Позиций#kkm_right#".count($receipt['receiptlines'])."#kkm_br#";

			if($discount_sum>0) {
				$strs.="Скидка#kkm_right#".number_format($discount_sum, 2, '.', '').'#kkm_br#';
			}
			$strs.='ЧЕК '.$receipt['id'].'#kkm_br#';
		}

		/*if($receipt['time']>0) {
			$strs.=date('d-m-Y H:i', $receipt['time']);
		}*/
		$strs.=";\n";

		//$strs.="Касса: ".$this->api_data['rm_id']."#kkm_br#";
		//$strs.="Кассир ".$this->f3->get('authorization_user.short_name').";\n";

		// Подвал

		$strs.="postitem;\n";
		if($this->sets['footer']) {

			$strs.=$this->sets['footer']."\n";
		}
		$strs.="postitem_end;\n";
		return $this->send_ajax($strs);
	}

	// Проверка кода маркировки
	function check_mark_fr($mark) {
		$strs='check_mark;'.str_replace(';', '{{U+003B}}', $mark).";";
	}

	// Замена переменных
	function replace_strs($strs) {
		$strs=str_replace('#kkm_rm#', $this->api_data['rm_id'], $strs);
		$user=$this->f3->get('authorization_user.short_name');
		$print_user=$user;
		if($this->sets['fix_user']) {
			$user=$this->sets['fix_user'];
		}
		$strs=str_replace('#kkm_user#', $user, $strs);
		$strs=str_replace('#print_user#', $print_user, $strs);
		$strs=str_replace('#date#', date('d.m.Y'), $strs);
		$strs=str_replace('#time#', date('H:i:s'), $strs);
		if($this->receipt) {
			$strs=str_replace('#receipt_number#', $this->receipt['id'], $strs);
			$strs=str_replace('#order_number#', $this->receipt['order_number'], $strs);
			$strs=str_replace('#big_order_number#', $this->ascii_str($this->receipt['order_number']), $strs);
			$strs=str_replace('#table_number#', $this->receipt['table_id'], $strs);
			$strs=str_replace('#room_number#', $this->receipt['table']['room'], $strs);
			$strs=str_replace('#table#', $this->receipt['table']['title'], $strs);
			$strs=str_replace('#room#', $this->receipt['table']['room_title'], $strs);
		}
		else {
			$strs=str_replace('#receipt_number#', '', $strs);
			$strs=str_replace('#table_number#', '', $strs);
			$strs=str_replace('#order_number#', '', $strs);
			$strs=str_replace('#room_number#', '', $strs);
			$strs=str_replace('#table#', '', $strs);
			$strs=str_replace('#room#', '', $strs);
		}
		if($this->discount_sum) {
			$strs=str_replace('#discount_sum#', number_format($this->discount_sum, 2, '.', ''), $strs);
		}
		else {
			$strs=str_replace('#discount_sum#', 0.00, $strs);
		}
		return $strs;
	}

	// Проверка маркированного товара
	function check_mark($mark) {
		$strs='check_mark;'.str_replace(';', '{{U+003B}}', $mark).";\n";
		return $this->send_ajax($strs);
	}

	// Отправка запроса
	function send_ajax($strs) {
		$strs=$this->replace_strs($strs);
		if($this->f3->get('GET.continue_print')==1) {
			$this->f3->clear('GET.continue_print');
			$this->continue_print();
			return;
		}
		if(!$this->sets['active']) return true;
		return $this->fr->send_ajax($strs);
	}

	// Состояние смены
	function change_status() {
		// closed - закрыта
        // opened - открыта
        // expired - истекла
		if(!$this->sets['active'] || !$this->sets['type']) return 'opened'; // Если ККМ не используется или используется чекопечатающий принтер
		$status=$this->fr->change_status();
		return $status;
	}
}
